﻿using ProjectAirFighter.CollectionGenericObject;
using ProjectAirFighter.Drawning;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectAirFighter.CollectionGenericObjects;

public abstract class AbstractCompany
{
    /// <summary>
    /// Размер места(ширина)
    /// </summary>
    public readonly int _placeSizeWidth = 140;
    /// <summary>
    /// Размер места(высота)
    /// </summary>
    public readonly int _placeSizeHeight = 160;

    /// <summary>
    /// Ширина окна
    /// </summary>
    protected readonly int _pictureWidth;

    /// <summary>
    /// Высота окна
    /// </summary>
    protected readonly int _pictureHeight;

    /// <summary>
    /// Коллекция военных самолетов
    /// </summary>
    protected ICollectionGenericObjects<DrawningWarPlane> _collection = null;

    /// <summary>
    /// Вычисление максимального количества элементов, которые можно разместить в окне
    /// </summary>
    private int GetMaxCount => _pictureWidth * _pictureHeight / (_placeSizeWidth * _placeSizeHeight);

    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="picWidth">Ширина окна</param>
    /// <param name="picHeight">Высота окна</param>
    /// <param name="collection">Коллекция военных самолетов</param>
    public AbstractCompany(int picWidth, int picHeight, ICollectionGenericObjects<DrawningWarPlane> collection)
    {
        _pictureWidth = picWidth;
        _pictureHeight = picHeight;
        _collection = collection;
        _collection.MaxCount = GetMaxCount - 4;
    }

    /// <summary>
    /// Перегрузка оператора сложения для класса
    /// </summary>
    /// <param name="company"></param>
    /// <param name="warPlane"></param>
    /// <returns></returns>
    public static int operator +(AbstractCompany company, DrawningWarPlane warPlane)
    {
        return company._collection.Insert(warPlane);
    }

    /// <summary>
    /// Перегрузка оператора удаление для класса
    /// </summary>
    /// <param name="company"></param>
    /// <param name="position"></param>
    /// <returns></returns>
    public static DrawningWarPlane operator -(AbstractCompany company, int position)
    {
        return company._collection.Remove(position);
    }

    /// <summary>
    /// Получение случайного объекта из коллекции
    /// </summary>
    /// <returns></returns>
    public DrawningWarPlane? GetRandomObject()
    {
        Random rnd = new();
        return _collection?.Get(rnd.Next(GetMaxCount));
    }
    /// <summary>
    /// Вывод всей коллекции
    /// </summary>
    /// <returns></returns>
    public Bitmap? Show()
    {
        Bitmap bitmap = new(_pictureWidth, _pictureHeight);
        Graphics graphics = Graphics.FromImage(bitmap);
        DrawBackgound(graphics);

        SetObjectsPosition();
        for (int i = 0; i < (_collection?.Count ?? 0); ++i)
        {
            DrawningWarPlane? obj = _collection?.Get(i);
            obj?.DrawTransport(graphics);
        }
        return bitmap;
    }

    /// <summary>
    /// Вывод заднего фона
    /// </summary>
    /// <param name="g"></param>
    protected abstract void DrawBackgound(Graphics g);
    /// <summary>
    /// Расстановка объектов
    /// </summary>
    protected abstract void SetObjectsPosition();

}
