﻿using System.Net.NetworkInformation;
using System.Net.Sockets;
using ProjectAirFighter.Entities;

namespace ProjectAirFighter.Drawning;

/// <summary>
/// класс отвечающий за прорисовку и перемещение объекта-сущности
/// </summary>
public class DrawningAirFighter : DrawningWarPlane
{

    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="bodyKit">Признак наличия корпуса</param>
    /// <param name="rocket">Признак наличия рокет</param>
    /// <param name="additionalWing">Признак наличия дополнительных крыльев</param>
    //public void Init(int speed, double weight, Color bodyColor, Color additionalColor, bool rocket, bool additionalWing)
    public DrawningAirFighter(int speed, double weight, Color bodyColor, Color additionalColor, bool rocket, bool additionalWing) : base(120,140)
    {
        EntityWarPlane = new EntityAirFighter(speed, weight, bodyColor, additionalColor, rocket, additionalWing);
        
        
    }

    /// <summary>
    /// Конструктор для метода создания объекта из строки (ExtentionDrawningWarPlane)
    /// </summary>
    /// <param name="warPlane"></param>
    public DrawningAirFighter(EntityWarPlane? warPlane) : base(warPlane)
    {
        if (warPlane != null)
            EntityWarPlane = warPlane;
    }
    
    public override void DrawTransport(Graphics g)
    {
        if (EntityWarPlane == null || EntityWarPlane is not EntityAirFighter warPlane||!_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }
        Pen pen = new(Color.Black, 3);
        Pen pen_rocket = new(Color.Black);

        Brush brAdd = new SolidBrush(warPlane.AdditionalColor);

        //ракеты
        if (warPlane.Rocket)
        {
            Point[] FirstRocket = { new Point(_startPosX.Value + 55, _startPosY.Value + 110), new Point(_startPosX.Value + 95, _startPosY.Value + 110),
                new Point(_startPosX.Value + 105, _startPosY.Value + 115),new Point(_startPosX.Value + 95, _startPosY.Value + 120),
                new Point(_startPosX.Value + 55, _startPosY.Value + 120) };
            g.DrawPolygon(pen_rocket, FirstRocket);
            Point[] SecondRocket = { new Point(_startPosX.Value + 55, _startPosY.Value + 30), new Point(_startPosX.Value + 95, _startPosY.Value + 30),
                new Point(_startPosX.Value + 105, _startPosY.Value + 35),new Point(_startPosX.Value + 95, _startPosY.Value + 40),
                new Point(_startPosX.Value + 55, _startPosY.Value + 40) };
            g.DrawPolygon(pen_rocket, SecondRocket);

            g.FillPolygon(brAdd, FirstRocket);
            g.FillPolygon(brAdd, SecondRocket);
        }
        base.DrawTransport(g);
        if (warPlane.AdditionalWing)
        {
            Point[] AddWingLower = { new Point(_startPosX.Value + 60, _startPosY.Value + 80),
            new Point(_startPosX.Value + 60, _startPosY.Value + 140),  new Point(_startPosX.Value + 50, _startPosY.Value + 140),
            new Point(_startPosX.Value + 40, _startPosY.Value + 80) };
            g.DrawPolygon(pen, AddWingLower);

            Point[] AddWingUpper =  { new Point(_startPosX.Value + 60, _startPosY.Value + 60), new Point(_startPosX.Value + 60, _startPosY.Value),
            new Point(_startPosX.Value + 50, _startPosY.Value), new Point(_startPosX.Value + 40, _startPosY.Value + 60) };
            g.DrawPolygon(pen, AddWingUpper);

            g.FillPolygon(brAdd, AddWingUpper);
            g.FillPolygon(brAdd, AddWingLower);
        }
        
    }

   

    
}
