﻿namespace ProjectAirFighter.Entities;
/// <summary>
/// Класс-сущность "Истребитель"
/// </summary>
public class EntityAirFighter : EntityWarPlane
{
    /// <summary>
    /// Дополнительный цвет
    /// </summary>
    public Color AdditionalColor { get; private set; }

    /// <summary>
    /// Признак (опция) наличия ракет
    /// </summary>
    public bool Rocket { get; private set; }

    /// <summary>
    /// Признак (опция) наличия дополнительных крыльев
    /// </summary>
    public bool AdditionalWing { get; private set; }

    /// <summary>
    /// Шаг
    /// </summary>
    public double Step => Speed * 100 / Weight;

    /// <summary>
    /// Иницифлизация полей объекта-класса истребителя
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="rocket">Признак наличия рокет</param>
    /// <param name="additionalWing">Признак наличия дополнительных крыльев</param>
    
    public void SetAdditionalColor(Color addirionalColor)
    {
        AdditionalColor = addirionalColor;
    }
    
    public EntityAirFighter(int speed, double weight, Color bodyColor, Color additionalColor, bool rocket, bool additionalWing) :base(speed, weight, bodyColor)
    {
        
        AdditionalColor = additionalColor;
        Rocket = rocket;
        AdditionalWing = additionalWing;

    }

    /// <summary>
    /// Получение строк со значениями свойств объекта класса-сущности
    /// </summary>
    /// <returns></returns>
    public override string[] GetStringRepresentation()
    {
        return new[] { nameof(EntityAirFighter), Speed.ToString(), Weight.ToString(), BodyColor.Name, AdditionalColor.Name, 
            Rocket.ToString(), AdditionalWing.ToString()};
    }


    /// <summary>
    /// Создание объекта из массива строк
    /// </summary>
    /// <param name="strs"></param>
    /// <returns></returns>
    public static EntityAirFighter? CreateEntityAirFighter(string[] strs)
    {
        if (strs.Length != 7 || strs[0] != nameof(EntityAirFighter))
        {
            return null;
        }

        return new EntityAirFighter(Convert.ToInt32(strs[1]), Convert.ToDouble(strs[2]), Color.FromName(strs[3]), Color.FromName(strs[4]), 
            Convert.ToBoolean(strs[5]), Convert.ToBoolean(strs[6]));
    }

}


