﻿namespace ProjectAirFighter.Entities;
/// <summary>
/// Класс-сущность "Военный самолет"
/// </summary>

public class EntityWarPlane
{
    /// <summary>
    /// Скорость 
    /// </summary>
    public int Speed { get; private set; }

    /// <summary>
    /// Вес
    /// </summary>
    public double Weight { get; private set; }

    /// <summary>
    /// Основной цвет
    /// </summary>
    public Color BodyColor { get; private set; }

    /// <summary>
    /// Шаг
    /// </summary>
    public double Step => Speed * 100 / Weight;

    public void SetBodyColor(Color bodyColor)
    {
        BodyColor = bodyColor;
    }
    
    /// <summary>
    /// Конструктор сущности
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="rocket">Признак наличия рокет</param>
    /// <param name="additionalWing">Признак наличия дополнительных крыльев</param>
    public EntityWarPlane(int speed, double weight, Color bodyColor)
    {
        Speed = speed;
        Weight = weight;
        BodyColor = bodyColor;
    }

    /// <summary>
    /// Получение строк со значениями свойств объекта класса-сущности
    /// </summary>
    /// <returns></returns>
    public virtual string[] GetStringRepresentation()
    {
        return new[] {nameof(EntityWarPlane),Speed.ToString(), Weight.ToString(), BodyColor.Name};
    }


    /// <summary>
    /// Создание объекта из массива строк
    /// </summary>
    /// <param name="strs"></param>
    /// <returns></returns>
    public static EntityWarPlane? CreateEntityWarPlane(string[] strs)
    {
        if (strs.Length != 4 || strs[0] != nameof(EntityWarPlane))
        {
            return null;
        }

        return new EntityWarPlane(Convert.ToInt32(strs[1]), Convert.ToDouble(strs[2]), Color.FromName(strs[3]));
    }


}
