﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ProjectAirFighter.Drawning;
using ProjectAirFighter.Entities;
using ProjectAirFighter.MovementStrategy;

namespace ProjectAirFighter
{
    public partial class FormAirFighter : Form


    {
        private DrawningWarPlane? _drawningWarPlane;

        /// <summary>
        /// Стратегия перемещения
        /// </summary>
        private AbstractStrategy? _strategy;
        /// <summary>
        /// Конструктор формы
        /// </summary>
        
        public DrawningWarPlane SetWarPlane{
            set
            {
                _drawningWarPlane = value;
                _drawningWarPlane.SetPictureSize(pictureBoxAirFighter.Width, pictureBoxAirFighter.Height);
                comboBoxStrategy.Enabled = true;
                _strategy = null;
                Draw();
            }
        }
        public FormAirFighter()
        {
            InitializeComponent();
            _strategy = null;
        }
        /// <summary>
        /// Метод прорисовки
        /// </summary>
        private void Draw()
        {
            if (_drawningWarPlane == null) return;

            Bitmap bmp = new(pictureBoxAirFighter.Width, pictureBoxAirFighter.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _drawningWarPlane.DrawTransport(gr);
            pictureBoxAirFighter.Image = bmp;
        }
       

        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if (_drawningWarPlane == null)
            {
                return;
            }

            string name = ((Button)sender)?.Name ?? string.Empty;
            bool result = false;
            switch (name)
            {
                case "buttonUp":
                    result = _drawningWarPlane.MoveTransport(DirectionType.Up);
                    break;
                case "buttonDown":
                    result = _drawningWarPlane.MoveTransport(DirectionType.Down);
                    break;
                case "buttonLeft":
                    result = _drawningWarPlane.MoveTransport(DirectionType.Left);
                    break;
                case "buttonRight":
                    result = _drawningWarPlane.MoveTransport(DirectionType.Right);
                    break;
            }
            if (result)
            {
                Draw();
            }


        }
        private void FormAirFighter_SizeChanged(object sender, EventArgs e)
        {
            if (_drawningWarPlane == null) return;


            _drawningWarPlane.SetPictureSize(pictureBoxAirFighter.Width, pictureBoxAirFighter.Height);


            if (_drawningWarPlane.SetPictureSize(pictureBoxAirFighter.Width, pictureBoxAirFighter.Height))
            {
                Draw();
            }

        }

        private void buttonStrategyStep_Click(object sender, EventArgs e)
        {
            if (_drawningWarPlane == null)
            {
                return;
            }
            if (comboBoxStrategy.Enabled)
            {
                _strategy = comboBoxStrategy.SelectedIndex switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToBorder(),
                    _ => null,
                };
                if (_strategy == null)
                {
                    return;
                }
                _strategy.SetDate(new MoveableWarPlane(_drawningWarPlane), pictureBoxAirFighter.Width, pictureBoxAirFighter.Height);
            }

            if (_strategy == null)
            {
                return;
            }

            comboBoxStrategy.Enabled = false;
            _strategy.MakeStep();
            Draw();

            if (_strategy.GetStatus() == StrategyStatus.Finish)
            {
                comboBoxStrategy.Enabled = true;
                _strategy = null;
            }
        }
    }
}
