﻿using ProjectAirFighter.Drawning;
using ProjectAirFighter.Entities;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectAirFighter;

/// <summary>
/// Форма конфигурации объекта
/// </summary>
public partial class FormWarPlaneConfig : Form
{

    /// <summary>
    /// Объект - прорисовка военного самолета
    /// </summary>
    private DrawningWarPlane _warPlane = null;

    /// <summary>
    /// Событие для передачи объекта
    /// </summary>
    private event Action<DrawningWarPlane>? WarPlaneDelegate;

    /// <summary>
    /// Конструктор
    /// </summary>
    public FormWarPlaneConfig()
    {
        InitializeComponent();

        panelRed.MouseDown += Panel_MouseDown;
        panelGreen.MouseDown += Panel_MouseDown;
        panelWhite.MouseDown += Panel_MouseDown;
        panelBlack.MouseDown += Panel_MouseDown;
        panelYellow.MouseDown += Panel_MouseDown;
        panelBlue.MouseDown += Panel_MouseDown;
        panelGrey.MouseDown += Panel_MouseDown;
        panelPurple.MouseDown += Panel_MouseDown;

        //TODO привязать анонимный метод через lamda с закрытием формы
        buttonCancel.Click += (sender, e) => Close();


    }


    /// <summary>
    /// Привязка внешнего метода к событию
    /// </summary>
    /// <param name="warPlaneDelegate"></param>
    public void AddEvent(Action<DrawningWarPlane> warPlaneDelegate)
    {
        if (WarPlaneDelegate == null)
        {
            WarPlaneDelegate = warPlaneDelegate;
        }
        else
        {
            WarPlaneDelegate += warPlaneDelegate;
        }
    }

    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>

    private void DrawObject()
    {
        Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _warPlane?.SetPictureSize(pictureBoxObject.Width, pictureBoxObject.Height);
        _warPlane?.SetPosition(5, 5);
        _warPlane?.DrawTransport(gr);
        pictureBoxObject.Image = bmp;
    }

    /// <summary>
    /// Передаем информацию при нажатии Label
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void LabelObject_MouseDown(object sender, MouseEventArgs e)
    {
        (sender as Label)?.DoDragDrop((sender as Label)?.Name ?? string.Empty, DragDropEffects.Move | DragDropEffects.Copy);
    }

    /// <summary>
    /// Проверка получаемой информации (ее типа на соответствие требуемому)
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void PanelObject_DragEnter(object sender, DragEventArgs e)
    {
        e.Effect = e.Data?.GetDataPresent(DataFormats.Text) ?? false ? DragDropEffects.Copy : DragDropEffects.None;
    }


    /// <summary>
    /// Действие при приеме претаскиваеиой информации
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void PanelObject_DragDrop(object sender, DragEventArgs e)
    {
        switch (e.Data?.GetData(DataFormats.Text)?.ToString())
        {
            case "labelSimpleObject":
                _warPlane = new DrawningWarPlane((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value, Color.White);
                break;
            case "labelModifiedObject":
                _warPlane = new DrawningAirFighter((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value, Color.White, Color.Black, checkBoxRocket.Checked, checkBoxAdditionalWing.Checked);
                break;
        }
        DrawObject();
    }

    private void Panel_MouseDown(object? sender, MouseEventArgs e)
    {
        //TODO
        (sender as Control)?.DoDragDrop((sender as Control)?.BackColor!, DragDropEffects.Move | DragDropEffects.Copy);
    }

    private void labelBodyColor_DragDrop(object sender, DragEventArgs e)
    {
        if (_warPlane == null) return;
        _warPlane.EntityWarPlane?.SetBodyColor((Color)e.Data.GetData(typeof(Color)));
        DrawObject();
    }

    private void labelBodyColor_DragEnter(object sender, DragEventArgs e)
    {
        if (e.Data.GetDataPresent(typeof(Color)))

            e.Effect = DragDropEffects.Copy;

        else
            e.Effect = DragDropEffects.None;
    }

    private void labelAdditionalColor_DragDrop(object sender, DragEventArgs e)
    {
        if (_warPlane?.EntityWarPlane is EntityAirFighter _airFighter)
        {
            _airFighter?.SetAdditionalColor((Color)e.Data.GetData(typeof(Color)));
        }
        DrawObject();
    }
    private void labelAdditionalColor_DragEnter(object sender, DragEventArgs e)
    {
        if (e.Data.GetDataPresent(typeof(Color)))
            e.Effect = DragDropEffects.Copy;
        else
            e.Effect = DragDropEffects.None;
    }

    private void buttonAdd_Click(object sender, EventArgs e)
    {
        if (_warPlane != null)
        {
            WarPlaneDelegate?.Invoke(_warPlane);
            Close();
        }
    }

   
}
